
#include "OleDragDropExpTreeViewCtrl.h"
#include "ChevronHandler.h"

// Options
#define ETV_EX_MYCOMPUTER			0x00010000L
#define ETV_EX_FAVORITES			0x00020000L
#define ETV_EX_FAVORITEGROUP		0x00040000L
#define ETV_EX_USERDEFINED			0x00080000L
#define ETV_EX_HISTORY				0x00100000L // UDT DGSTR

class CFavoritesTreeViewCtrl : 
	public COleDragDrogExplorerTreeViewCtrlImpl<CFavoritesTreeViewCtrl>
{
public:
	DECLARE_WND_SUPERCLASS(_T("Mtl_FavTreeViewCtrl"), GetWndClassName())
	typedef COleDragDrogExplorerTreeViewCtrlImpl<CFavoritesTreeViewCtrl> baseClass;
	typedef CFavoritesTreeViewCtrl thisClass;

// Constants
	enum { s_nIDOpenAll = 10, };

// Ctor
	CFavoritesTreeViewCtrl() : m_dwStyle(ETV_EX_FAVORITES)
	{
		_GetProfile();
	}

	~CFavoritesTreeViewCtrl()
	{
		_WriteProfile();
	}

// Attributes
	DWORD m_dwStyle;

	bool _IsNeedFavoritesFix()
	{
		if (_check_flag(ETV_EX_FAVORITES, m_dwStyle) || _check_flag(ETV_EX_USERDEFINED, m_dwStyle))
			return true;
		else
			return false;
	}

// Overrides
	CItemIDList OnInitRootFolder()
	{
		if (_check_flag(ETV_EX_MYCOMPUTER, m_dwStyle)) {
			CItemIDList idl;
			HRESULT hr = ::SHGetSpecialFolderLocation(m_hWnd, CSIDL_DESKTOP, &idl);
			ATLASSERT(SUCCEEDED(hr));
			return idl;
		}
		else if (_check_flag(ETV_EX_FAVORITEGROUP, m_dwStyle)) {
			CString strDir = DonutGetFavoriteGroupFolder();
			return strDir;
		}
		else if (_check_flag(ETV_EX_USERDEFINED, m_dwStyle)) {
			return CFavoritesMenuOption::GetUserDirectory();
		}
		// UDT DGSTR
		else if (_check_flag(ETV_EX_HISTORY, m_dwStyle)) {
			CString strDir;
			CItemIDList idl;
			if(MtlGetHistoryFolder(strDir))
				return strDir;
			else {
			HRESULT hr = ::SHGetSpecialFolderLocation(m_hWnd, CSIDL_HISTORY, &idl);
			ATLASSERT(SUCCEEDED(hr));
			return idl;
			}
		}
		else {
			CString strDir; MtlGetFavoritesFolder(strDir);
			return strDir;
		}
	}

	void OnInitialUpdateTreeItems(CSimpleArray<TV_INSERTSTRUCT>& items, LPCITEMIDLIST pidl)
	{
		ATLASSERT(pidl != NULL);

		DWORD dwStyle = CFavoritesMenuOption::GetStyle();
		if (_check_flag(EMS_ORDER_FAVORITES, dwStyle) && _check_flag(ETV_EX_FAVORITES, m_dwStyle)) {
			CFavoritesOrder order;
			MtlGetFavoritesOrder(order, CItemIDList(pidl).GetPath());
			std::sort(_begin(items), _end(items), _FavoritesTreeItemCompare<thisClass>(order, this));
		}
		else if (_check_flag(ETV_EX_HISTORY, m_dwStyle)){} // UDT DGSTR
		else {
			baseClass::OnInitialUpdateTreeItems(items, pidl);
		}
	}

//  override or not
//	void OnOpenTreeItem(HTREEITEM hItem, LPITEMIDLIST pidl)
//	{
//		if (_check_flag(ETV_EX_MYCOMPUTER, m_dwStyle)) {
//			MtlShellExecute(m_hWnd, pidl);
//			return;
//		}
//
//		CItemIDList idl(pidl);
//		if (!idl.IsNull())
//			DonutOpenFile(m_hWnd, idl.GetPath(), DonutGetStdOpenFlag());
//	}

	void OnTreeItemClicked(HTREEITEM hTreeItem, UINT uFlags)
	{
		if (_check_flag(ETV_EX_MYCOMPUTER, m_dwStyle)) {
			MtlShellExecute(m_hWnd, _GetItemIDList(hTreeItem));
			return;
		}
		// UDT DGSTR
		else if (_check_flag(ETV_EX_HISTORY, m_dwStyle)) {
			/* whats a hell IPersistFile::Load..*/
			MtlShellExecute(m_hWnd, _GetItemIDList(hTreeItem));
			return;
		}
		// ENDE
		CItemIDList idl = _GetItemIDList(hTreeItem);
		if (!idl.IsNull())
			DonutOpenFile(m_hWnd, idl.GetPath(), DonutGetStdOpenFlag());
	}

	bool OnGetInfoToolTip(HTREEITEM hItem, CString& strTip, int cchTextMax)
	{
		if (_IsItemFolder(hItem))
			return false;

		CItemIDList idl = _GetItemIDList(hItem);
		if (idl.IsNull())
			return false;

		CString strPath = idl.GetPath();
		if (strPath.IsEmpty())
			return false;


		CString strUrl = MtlIsExt(strPath, _T(".url")) ? MtlGetInternetShortcut(strPath) : CString(); // UDT DGSTR ( fix )
		if (strUrl.IsEmpty())
			strUrl = strPath;

		CString strName;
		if (!GetItemText(hItem, strName))
			return false;

		strTip = MtlMakeFavoriteToolTipText(strName, strUrl, cchTextMax);
		return true;
	}

	bool OnCmdContextMenu(HTREEITEM hTreeItem, int nID)
	{
		if (nID != s_nIDOpenAll)
			return false;

		MtlForEachFile(_GetItemIDList(hTreeItem), _Function_OpenFile(m_hWnd));
		return true;
	}

	struct _Function_OpenFile
	{
		HWND _hWnd;

		_Function_OpenFile(HWND hWnd) : _hWnd(hWnd) { }

		void operator()(IShellFolder* pFolder, const CItemIDList& idlFolder, const CItemIDList& idlFile)
		{
			CItemIDList idlFull = idlFolder + idlFile;
			if (!idlFull.IsNull())
				DonutOpenFile(_hWnd, idlFull.GetPath());
		}
	};

	UINT OnPrepareMenuForContext(HTREEITEM hTreeItem, CMenuHandle menu, int& nPos)
	{
		bool bFolder = _IsItemFolder(hTreeItem);
		UINT uFlags = CMF_CANRENAME;

		if (bFolder)
			uFlags |= CMF_EXPLORE | CMF_NODEFAULT;
		else
			uFlags |= CMF_NORMAL;

		if (!_IsNeedFavoritesFix())
			return uFlags;

		// set up for favorites
		if (bFolder) {
			if (_IsItemExpanded(hTreeItem))
				menu.AppendMenu(MF_ENABLED|MF_STRING, s_nIDCollapse, _T("܂肽(&A)"));
			else
				menu.AppendMenu(MF_ENABLED|MF_STRING, s_nIDExpand, _T("WJ(&A)"));

			menu.AppendMenu(MF_ENABLED|MF_STRING, s_nIDOpenAll, _T("gSĊJ(&I)"));
			menu.AppendMenu(MF_ENABLED|MF_STRING, s_nIDNewFolder, _T("VtH_쐬(&W)"));
			menu.AppendMenu(MF_SEPARATOR);
			nPos += 4;
		}
		else {

		}

		return uFlags;
	}

	void OnPrepareMenuForPopup(HTREEITEM hTreeItem, CMenuHandle menu)
	{
		if (!_IsNeedFavoritesFix())
			return;

		int nCmdID = MtlGetCmdIDFromAccessKey(menu, _T("&S"));// remove "create shortcut" menu item
		if (nCmdID != -1)
			menu.RemoveMenu(nCmdID, MF_BYCOMMAND);

		if (!_IsItemFolder(hTreeItem)) {
			int nCmdID = MtlGetCmdIDFromAccessKey(menu, _T("&O"), false);// remove original "open" menu item
			if (nCmdID != -1)
				menu.RemoveMenu(nCmdID, MF_BYCOMMAND);

			menu.InsertMenu(0, MF_BYPOSITION|MF_STRING, s_nIDNewFolder, _T("VtH_쐬(&W)"));
			menu.InsertMenu(0, MF_BYPOSITION|MF_STRING, s_nIDOpen, _T("J(&O)"));

			menu.SetMenuDefaultItem(s_nIDOpen, FALSE);
		}
	}

	template <typename _This>
	struct _FavoritesTreeItemCompare : public std::binary_function<const TV_INSERTSTRUCT&, const TV_INSERTSTRUCT&, bool>
	{
		CFavoritesOrder& _order;
		_This* _pThis;

		_FavoritesTreeItemCompare(CFavoritesOrder& order, _This* pThis) : _order(order), _pThis(pThis) { }

		bool operator()(const TV_INSERTSTRUCT& x, const TV_INSERTSTRUCT& y)
		{
			CItemIDList idlA = (LPCITEMIDLIST)x.item.lParam; ATLASSERT(!idlA.IsNull());
			CItemIDList idlB = (LPCITEMIDLIST)y.item.lParam; ATLASSERT(!idlB.IsNull());

			const int enough = 10000;
			CString strA = MtlGetFileName(idlA.GetPath());
			CString strB = MtlGetFileName(idlB.GetPath());
			ATLASSERT(!strA.IsEmpty() && !strB.IsEmpty());
			
			int itA = _order.Lookup(strA);
			int itB = _order.Lookup(strB);
			if (itA == -1)
				itA = enough;
			if (itB == -1)
				itB = enough;

			if (itA == enough && itB == enough) {
				bool bParentA = (x.item.cChildren == 1);
				bool bParentB = (y.item.cChildren == 1);
				if (bParentA == bParentB) {
					return strA < strB;
				}
				else {
					if (bParentA)
						return true;
					else
						return false;
				}
			}
			else 
				return itA < itB;
		}

		CString _GetFileName(const TV_INSERTSTRUCT& tvi)
		{
			bool bDir = (tvi.item.cChildren == 1);
			if (bDir) {
				return tvi.item.pszText;
			}

			CItemIDList idl = (LPCITEMIDLIST)tvi.item.lParam;
			CString str = idl.GetPath();

			if (str.IsEmpty())
				return CString();
			
			int nIndex = str.ReverseFind(_T('\\'));
			str = str.Right(str.GetLength() - nIndex - 1);
	
			return str;
		}

		CString _GetFilePath(const TV_INSERTSTRUCT& tvi)
		{
			CItemIDList idl = (LPCITEMIDLIST)tvi.item.lParam;
			CString str = idl.GetPath();
			return str;
		}

		FILETIME _GetFileTime(const CString& strPath_)
		{
			CString strPath(strPath_);
			MtlRemoveTrailingBackSlash(strPath);
			WIN32_FIND_DATA data;
			HANDLE h = ::FindFirstFile(strPath, &data);
			ATLASSERT(h != INVALID_HANDLE_VALUE);
			::FindClose(h);

			return data.ftCreationTime;
		}
	};

	void _GetProfile()
	{
		CIniSection pr;
		CString strSection = _T("FavTree");

		pr.Open(_szIniFileName, strSection);
		pr.QueryValue(m_dwStyle, _T("Style"));
		pr.Close();
	}

	void _WriteProfile()
	{
		CIniSection pr;
		CString strSection = _T("FavTree");

		pr.Open(_szIniFileName, strSection);
		pr.SetValue(m_dwStyle, _T("Style"));
		pr.Close();
	}
};


template <class T, class TBase = CWindow, class TWinTraits = CControlWinTraits>
class CDonutFavoritesBarImpl : 
	public CWindowImpl< T, TBase, TWinTraits >,
	public CChevronHandler<CDonutFavoritesBarImpl>,
	public CUpdateCmdUI<CDonutFavoritesBarImpl>
{
public:
	DECLARE_WND_CLASS_EX(NULL, 0, -1)
	typedef CDonutFavoritesBarImpl<T, TBase, TWinTraits> thisClass;

// Constants
	enum
	{
		s_cxyBorder = 2,
		s_cxyBtnOffset = 1,
		s_cxyViewGap = 1,
	};

// Data members
	CToolBarCtrl m_wndToolBar;
	CReBarCtrl	m_wndReBar;
	CFavoritesTreeViewCtrl m_view;
	int m_cxyHeader;
	DWORD m_dwStyle;

	CMenu m_menuPlacement;

// Ctor
	CDonutFavoritesBarImpl() : m_dwStyle(ETV_EX_FAVORITES)
	{
		m_menuPlacement.LoadMenu(IDR_MENU_FAVTREE_BAR);
	}

// Overrides
	HMENU OnGetDropDownMenu(int nCmdID)
	{
		return m_menuPlacement.GetSubMenu(0);//.m_hMenu;
	}

// Methods
	BYTE PreTranslateMessage(MSG* pMsg)
	{
		UINT msg = pMsg->message;
		int vKey =  pMsg->wParam;

		if (msg == WM_SYSKEYDOWN || msg == WM_SYSKEYUP || msg == WM_KEYDOWN) {

			if (pMsg->hwnd == m_view.GetEditControl())
				return _MTL_TRANSLATE_WANT;// pass to rename edit ctrl

			if (pMsg->hwnd != m_view.m_hWnd || !m_view.IsWindowVisible())
				return _MTL_TRANSLATE_PASS;

			if (vKey == VK_LEFT || vKey == VK_RIGHT || vKey == VK_DOWN || vKey == VK_UP ||
				vKey == VK_RETURN || vKey == VK_SPACE) {
				return _MTL_TRANSLATE_WANT;// pass to control
			}
		}

		return _MTL_TRANSLATE_PASS;
	}

	// Message map and handlers
	BEGIN_MSG_MAP(CDonutFavoritesBarImpl)
		MESSAGE_HANDLER(WM_CREATE, OnCreate)
		MESSAGE_HANDLER(WM_SIZE, OnSize)
		MESSAGE_HANDLER(WM_SETFOCUS, OnSetFocus)
		MESSAGE_HANDLER(WM_DESTROY, OnDestroy)
		MESSAGE_HANDLER(WM_PAINT, OnPaint)
		MESSAGE_HANDLER(WM_ERASEBKGND, OnEraseBackground)

		COMMAND_ID_HANDLER(ID_FAVORITE_PLACEMENT, OnFavoritePlacement)
		COMMAND_ID_HANDLER(ID_VIEW_REFRESH_FAVBAR, OnFavoritePlacement)

		COMMAND_ID_HANDLER(ID_FAVTREE_BAR_STANDARD, OnFavTreeStandard)
		COMMAND_ID_HANDLER(ID_FAVTREE_BAR_GROUP, OnFavTreeGroup)
		COMMAND_ID_HANDLER(ID_FAVTREE_BAR_USER, OnFavTreeUser)
		COMMAND_ID_HANDLER(ID_FAVTREE_BAR_MYCOMPUTER, OnFavTreeMyComputer)
		
		// UDT DGSTR
		COMMAND_ID_HANDLER(ID_FAVTREE_BAR_HISTORY, OnFavTreeHistory)
		// ENDE

		NOTIFY_CODE_HANDLER(TBN_DROPDOWN, OnTbnDropDown)

		CHAIN_MSG_MAP(CChevronHandler<CDonutFavoritesBarImpl>)
		CHAIN_MSG_MAP(CUpdateCmdUI<CDonutFavoritesBarImpl>)
		FORWARD_NOTIFICATIONS()
	END_MSG_MAP()

	LRESULT OnFavTreeStandard(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		m_view.m_dwStyle = ETV_EX_FAVORITES;
		
		SendMessage(GetTopLevelParent(), WM_UPDATE_EXPBAR, (WPARAM)_T("Cɓ"), (LPARAM)0); // UDT DGSTR
		m_wndToolBar.UpdateWindow();
		m_view.RefreshRootTree();
		return 0;
	}
	LRESULT OnFavTreeGroup(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		m_view.m_dwStyle = ETV_EX_FAVORITEGROUP;
		
		SendMessage(GetTopLevelParent(), WM_UPDATE_EXPBAR, (WPARAM)_T("CɓO[v"), (LPARAM)0); // UDT DGSTR

		m_wndToolBar.UpdateWindow();
		m_view.RefreshRootTree();
		return 0;
	}
	LRESULT OnFavTreeUser(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		m_view.m_dwStyle = ETV_EX_USERDEFINED;

		SendMessage(GetTopLevelParent(), WM_UPDATE_EXPBAR, (WPARAM)_T("[U`"), (LPARAM)0); // UDT DGSTR

		m_wndToolBar.UpdateWindow();
		m_view.RefreshRootTree();

		return 0;
	}
	LRESULT OnFavTreeMyComputer(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		m_view.m_dwStyle = ETV_EX_MYCOMPUTER;

		SendMessage(GetTopLevelParent(), WM_UPDATE_EXPBAR, (WPARAM)_T("fXNgbv"), (LPARAM)0); // UDT DGSTR

		m_wndToolBar.UpdateWindow();
		m_view.RefreshRootTree();

		return 0;
	}
	// UDT DGSTR
	LRESULT OnFavTreeHistory(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		m_view.m_dwStyle = ETV_EX_HISTORY;

		SendMessage(GetTopLevelParent(), WM_UPDATE_EXPBAR, (WPARAM)_T(""), (LPARAM)0); // UDT DGSTR

		m_wndToolBar.UpdateWindow();
		m_view.RefreshRootTree();

		return 0;
	}
	// ENDE
	LRESULT OnFavoritePlacement(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		m_wndToolBar.UpdateWindow();
		m_view.RefreshRootTree();
		return 0;
	}

	LRESULT OnTbnDropDown(int /*idCtrl*/, LPNMHDR pnmh, BOOL& bHandled)
	{
		LPNMTOOLBAR lpnmtb = (LPNMTOOLBAR)pnmh;

		CRect rc; m_wndToolBar.GetItemRect(0, &rc);
		ClientToScreen(&rc);
		UINT uMenuFlags = TPM_LEFTBUTTON | TPM_RIGHTBUTTON | TPM_VERTICAL | TPM_LEFTALIGN | TPM_TOPALIGN | (!AtlIsOldWindows() ? TPM_VERPOSANIMATION : 0);
		TPMPARAMS TPMParams;
		TPMParams.cbSize = sizeof(TPMPARAMS);
		TPMParams.rcExclude = rc;

		::TrackPopupMenuEx(m_menuPlacement.GetSubMenu(0), uMenuFlags, rc.left + 1, rc.bottom + 2, m_hWnd, &TPMParams);

		return TBDDRET_DEFAULT;
	}

	LRESULT OnPaint(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		CPaintDC dc(m_hWnd);

		T* pT = static_cast<T*>(this);
		pT->DrawPaneTitle(dc.m_hDC);

		return 0;
	}

	LRESULT OnEraseBackground(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		return 1;	// no background needed
	}

	LRESULT OnDestroy(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled)
	{
		bHandled = FALSE;
		MtlDestroyImageLists(m_hWnd);
		return 0;
	}

	LRESULT OnCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		m_view.Create(m_hWnd, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN |
			TVS_INFOTIP | TVS_EDITLABELS |TVS_SHOWSELALWAYS | TVS_SINGLEEXPAND | TVS_FULLROWSELECT | TVS_TRACKSELECT, 0);//WS_EX_STATICEDGE);

		m_wndReBar = _CreateSimpleReBarCtrl(m_hWnd, ATL_SIMPLE_REBAR_NOBORDER_STYLE|CCS_NOPARENTALIGN);

		CFavoritesMenuOption::Add(m_hWnd);

		return 0;
	}

	LRESULT OnSize(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& /*bHandled*/)
	{
		UpdateLayout(GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam));
		return 0;
	}

	LRESULT OnSetFocus(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		if(m_view.m_hWnd != NULL)
			m_view.SetFocus();
		return 0;
	}

	void InitToolBar(int nIDAdd, int nIDOrganize, int nIDSpecial,
					 UINT nImageBmpID, UINT nHotImageBmpID, 
					 int cx, int cy, COLORREF clrMask, UINT nFlags = ILC_COLOR8)
	{
		CImageList imgs;
		MTLVERIFY(imgs.Create(cx, cy, nFlags | ILC_MASK, 1, 1));

		CBitmap bmp;
		MTLVERIFY(bmp.LoadBitmap(nImageBmpID));
		imgs.Add(bmp, clrMask);

		CImageList imgsHot;
		MTLVERIFY(imgsHot.Create(cx, cy, nFlags | ILC_MASK, 1, 1));

		CBitmap bmpHot;
		MTLVERIFY(bmpHot.LoadBitmap(nHotImageBmpID));
		imgsHot.Add(bmpHot, clrMask);

		m_wndToolBar.Create(m_hWnd, rcDefault, NULL, /*ATL_SIMPLE_WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS | 
			CCS_NOMOVEY | CCS_NODIVIDER | CCS_NORESIZE | CCS_NOPARENTALIGN |
			TBSTYLE_FLAT | */ ATL_SIMPLE_TOOLBAR_PANE_STYLE | TBSTYLE_LIST, 0);
		m_wndToolBar.SetExtendedStyle(TBSTYLE_EX_DRAWDDARROWS);
		ATLASSERT(m_wndToolBar.IsWindow());
		m_wndToolBar.SetButtonStructSize(sizeof(TBBUTTON));

		m_wndToolBar.SetImageList(imgs);
		m_wndToolBar.SetHotImageList(imgsHot);

		// add Add and Organize button
		TBBUTTON btnSp = { 2, nIDSpecial, TBSTATE_ENABLED, TBSTYLE_BUTTON | TBSTYLE_AUTOSIZE | TBSTYLE_DROPDOWN, 0, 0 };
		MTLVERIFY(m_wndToolBar.InsertButton(-1, &btnSp));
		TBBUTTON btnAdd = { 0, nIDAdd, TBSTATE_ENABLED, TBSTYLE_BUTTON | TBSTYLE_AUTOSIZE, 0, 0 };
		MTLVERIFY(m_wndToolBar.InsertButton(-1, &btnAdd));
		TBBUTTON btnOrg = { 1, nIDOrganize, TBSTATE_ENABLED, TBSTYLE_BUTTON | TBSTYLE_AUTOSIZE, 0, 0 };
		MTLVERIFY(m_wndToolBar.InsertButton(-1, &btnOrg));

		CVersional<TBBUTTONINFO> bi;
		bi.dwMask = TBIF_TEXT;
		bi.pszText = _T("ǉ...");
		MTLVERIFY(m_wndToolBar.SetButtonInfo(nIDAdd, &bi));
		bi.pszText = _T("...");
		MTLVERIFY(m_wndToolBar.SetButtonInfo(nIDOrganize, &bi));
		bi.pszText = _T("ꏊ");
		MTLVERIFY(m_wndToolBar.SetButtonInfo(nIDSpecial, &bi));

		m_wndToolBar.AddStrings(_T("NS\0"));	// for proper item height

		_AddSimpleReBarBandCtrl(m_wndReBar, m_wndToolBar);

		CalcSize();
	}

	void UpdateLayout(int cxWidth, int cyHeight)
	{
		ATLASSERT(::IsWindow(m_hWnd));
		RECT rect;

		::SetRect(&rect, 0, 0, cxWidth, m_cxyHeader);
		if(m_wndReBar.m_hWnd != NULL)
			m_wndReBar.SetWindowPos(NULL, rect.left + s_cxyBorder, s_cxyBorder, rect.right - s_cxyBorder*2, rect.bottom, SWP_NOZORDER);

		if(m_view.m_hWnd != NULL) {
			CRect rc(rect.left, m_cxyHeader, cxWidth, cyHeight);
			rc.top += 1;
			rc.left += 1;
			rc.right -= 2;
			rc.bottom -= 2;
			m_view.SetWindowPos(NULL, rc, SWP_NOZORDER);
		}
		else
			rect.bottom = cyHeight;

		// I can't get the reason why I have to update now without InvalidateRect.
//		InvalidateRect(&rect); can't update a rect, correctly.
		CClientDC dc(m_hWnd);
		T* pT = static_cast<T*>(this);
		pT->DrawPaneTitle(dc.m_hDC);
	}

	void CalcSize()
	{
		CRect rcBtn;
		m_wndToolBar.GetItemRect(0, &rcBtn);
		m_cxyHeader = rcBtn.Height() + s_cxyBorder;
	}

	void DrawPaneTitle(CDCHandle dc)
	{
		CRect rect;
		GetClientRect(&rect);

//		CRect rc; m_view.GetWindowRect(&rc);
//		ScreenToClient(&rc);
		CPen pen; pen.CreatePen(PS_SOLID, 1, ::GetSysColor(COLOR_3DSHADOW));
		POINT pts[] = { { rect.right - 3, rect.top + m_cxyHeader }, { rect.left, rect.top + m_cxyHeader },
		{ rect.left, rect.bottom - 2 } };//, { rect.right - 3, rect.bottom - 1 } };

//		CRect rect_(rect);
//		rect_.top += m_cxyHeader;
//		dc.DrawEdge(&rect_, BDR_RAISEDINNER, BF_RECT | BF_FLAT);
		
//		POINT pts[] = { { rc.right - 1, rc.top - 1}, { rc.left, rc.top - 1},
//			{ rc.left, rc.bottom }, { rc.right - 1, rc.bottom } };
		HPEN hOldPen = dc.SelectPen(pen);
		dc.Polyline(pts, _countof(pts));
		dc.SelectPen(hOldPen);

		dc.DrawEdge(CRect(rect.left, rect.top + m_cxyHeader, rect.right, rect.bottom),
			EDGE_ETCHED, BF_RIGHT | BF_BOTTOM);

//		dc.DrawEdge(CRect(rect.left, rect.bottom - 2, rect.right - 2, rect.bottom + 10), EDGE_ETCHED, BF_TOP);

		rect.bottom = rect.top + m_cxyHeader;
		dc.DrawEdge(&rect, EDGE_ETCHED, BF_LEFT | BF_TOP | BF_RIGHT | BF_ADJUST);
		dc.FillRect(&rect, (HBRUSH)LongToPtr(COLOR_3DFACE + 1));
	}


	static HWND _CreateSimpleReBarCtrl(HWND hWndParent, DWORD dwStyle = ATL_SIMPLE_REBAR_STYLE, UINT nID = ATL_IDW_TOOLBAR)
	{
		// Ensure style combinations for proper rebar painting
		if(dwStyle & CCS_NODIVIDER && dwStyle & WS_BORDER)
			dwStyle &= ~WS_BORDER;
		else if(!(dwStyle & WS_BORDER) && !(dwStyle & CCS_NODIVIDER))
			dwStyle |= CCS_NODIVIDER;

		// Create rebar window
		HWND hWndReBar = ::CreateWindowEx(0, REBARCLASSNAME, NULL, dwStyle, 0, 0, 100, 100, hWndParent, (HMENU)LongToHandle(nID), _Module.GetModuleInstance(), NULL);
		if(hWndReBar == NULL)
		{
			ATLTRACE2(atlTraceUI, 0, _T("Failed to create rebar.\n"));
			return NULL;
		}

		// Initialize and send the REBARINFO structure
		REBARINFO rbi;
		rbi.cbSize = sizeof(REBARINFO);
		rbi.fMask  = 0;
		if(!::SendMessage(hWndReBar, RB_SETBARINFO, 0, (LPARAM)&rbi))
		{
			ATLTRACE2(atlTraceUI, 0, _T("Failed to initialize rebar.\n"));
			::DestroyWindow(hWndReBar);
			return NULL;
		}

		return hWndReBar;
	}

	static BOOL _AddSimpleReBarBandCtrl(HWND hWndReBar, HWND hWndBand, int nID = 0, LPTSTR lpstrTitle = NULL, BOOL bNewRow = FALSE, int cxWidth = 0, BOOL bFullWidthAlways = FALSE)
	{
		ATLASSERT(::IsWindow(hWndReBar));	// must be already created
#ifdef _DEBUG
		// block - check if this is really a rebar
		{
			TCHAR lpszClassName[sizeof(REBARCLASSNAME)];
			::GetClassName(hWndReBar, lpszClassName, sizeof(REBARCLASSNAME));
			ATLASSERT(lstrcmp(lpszClassName, REBARCLASSNAME) == 0);
		}
#endif //_DEBUG
		ATLASSERT(::IsWindow(hWndBand));	// must be already created

		// Get number of buttons on the toolbar
		int nBtnCount = (int)::SendMessage(hWndBand, TB_BUTTONCOUNT, 0, 0L);

		// Set band info structure
		REBARBANDINFO rbBand;
		rbBand.cbSize = sizeof(REBARBANDINFO);
#if (_WIN32_IE >= 0x0400)
		rbBand.fMask = RBBIM_CHILD | RBBIM_CHILDSIZE | RBBIM_STYLE | RBBIM_ID | RBBIM_SIZE | RBBIM_IDEALSIZE;
#else
		rbBand.fMask = RBBIM_CHILD | RBBIM_CHILDSIZE | RBBIM_STYLE | RBBIM_ID | RBBIM_SIZE;
#endif //!(_WIN32_IE >= 0x0400)
		if(lpstrTitle != NULL)
			rbBand.fMask |= RBBIM_TEXT;
		rbBand.fStyle = 0;
#if (_WIN32_IE >= 0x0500)
		if(nBtnCount > 0)	// add chevron style for toolbar with buttons
			rbBand.fStyle |= RBBS_USECHEVRON;
#endif //(_WIN32_IE >= 0x0500)
		if(bNewRow)
			rbBand.fStyle |= RBBS_BREAK;

		rbBand.lpText = lpstrTitle;
		rbBand.hwndChild = hWndBand;
		if(nID == 0)	// calc band ID
			nID = ATL_IDW_BAND_FIRST + (int)::SendMessage(hWndReBar, RB_GETBANDCOUNT, 0, 0L);
		rbBand.wID = nID;

		// Calculate the size of the band
		BOOL bRet;
		RECT rcTmp;
		if(nBtnCount > 0)
		{
			bRet = (BOOL)::SendMessage(hWndBand, TB_GETITEMRECT, nBtnCount - 1, (LPARAM)&rcTmp);
			ATLASSERT(bRet);
			rbBand.cx = (cxWidth != 0) ? cxWidth : rcTmp.right;
			rbBand.cyMinChild = rcTmp.bottom - rcTmp.top;
			rbBand.cxMinChild = 0;
/*			if(bFullWidthAlways)
			{
				rbBand.cxMinChild = rbBand.cx;
			}
			else if(lpstrTitle == 0)
			{
				bRet = (BOOL)::SendMessage(hWndBand, TB_GETITEMRECT, 0, (LPARAM)&rcTmp);
				ATLASSERT(bRet);
				rbBand.cxMinChild = rcTmp.right;
			}
			else
			{
				rbBand.cxMinChild = 0;
			}
*/
		}
		else	// no buttons, either not a toolbar or really has no buttons
		{
			bRet = ::GetWindowRect(hWndBand, &rcTmp);
			ATLASSERT(bRet);
			rbBand.cx = (cxWidth != 0) ? cxWidth : (rcTmp.right - rcTmp.left);
			rbBand.cxMinChild = (bFullWidthAlways) ? rbBand.cx : 0;
			rbBand.cyMinChild = rcTmp.bottom - rcTmp.top;
		}

#if (_WIN32_IE >= 0x0400)
		rbBand.cxIdeal = rbBand.cx;
#endif //(_WIN32_IE >= 0x0400)

		// Add the band
		LRESULT lRes = ::SendMessage(hWndReBar, RB_INSERTBAND, (WPARAM)-1, (LPARAM)&rbBand);
		if(lRes == 0)
		{
			ATLTRACE2(atlTraceUI, 0, _T("Failed to add a band to the rebar.\n"));
			return FALSE;
		}

		return TRUE;
	}

	BEGIN_UPDATE_COMMAND_UI_MAP(CDonutFavoritesBarImpl)
		UPDATE_COMMAND_UI_SETCHECK_FLAG(ID_FAVTREE_BAR_MYCOMPUTER, ETV_EX_MYCOMPUTER, m_view.m_dwStyle)
		UPDATE_COMMAND_UI_SETCHECK_FLAG(ID_FAVTREE_BAR_STANDARD, ETV_EX_FAVORITES, m_view.m_dwStyle)
		UPDATE_COMMAND_UI_SETCHECK_FLAG(ID_FAVTREE_BAR_GROUP, ETV_EX_FAVORITEGROUP, m_view.m_dwStyle)
		UPDATE_COMMAND_UI_SETCHECK_FLAG(ID_FAVTREE_BAR_USER, ETV_EX_USERDEFINED, m_view.m_dwStyle)
		UPDATE_COMMAND_UI_SETCHECK_FLAG(ID_FAVTREE_BAR_HISTORY, ETV_EX_HISTORY, m_view.m_dwStyle) // UDT DGSTR
	END_UPDATE_COMMAND_UI_MAP()
};

class CDonutFavoritesBar : public CDonutFavoritesBarImpl<CDonutFavoritesBar>
{
public:
	DECLARE_WND_CLASS_EX(_T("Donut_FavoritesBar"), 0, -1)
};
